 #include <PID_v1.h>       //library used to compute the PID 
#include <UTFT_tinyFAT.h> //libraries needed to use the touch screen
#include <UTouch.h>
#include <UTFT.h>
#include <filters.h>    //library needed to filter the output signal


#define TOUCH_ORIENTATION  LANDSCAPE   //set the orientation and the colors
#define BLACK   0x0000
#define BLUE    0x001F
#define RED     0xF800
#define GREEN   0x07E0
#define CYAN    0x07FF
#define MAGENTA 0xF81F
#define YELLOW  0xFFE0
#define WHITE   0xFFFF

UTFT lcd(TFT01_50,38,39,40,41);   //initialize the touch screen
UTFT_tinyFAT myFiles(&lcd);
UTouch  myTouch( 6, 5, 4, 3, 2);


extern uint8_t SmallFont[];  //set the fonts
extern uint8_t BigFont[];


float lista[400];     //array in which we will save the values generated by the wave equation
float maxValor=0;     //varible needed to compare the values and get the maximum
float minValor=300;   //varible needed to compare the values and get the minimim, we need a high value so it gets compared with every component of the list
int inputplotlast[760]; //array in which we will save the signal values ready to be plotted

float consigna;  

int i=0;  
float t=0, y;
float Tm =10;     //sampling time in miliseconds
float tiempo =0;
float t0=0;
int mu=95;   
int a1=10;
int a2=9;
float phase=1.2;
double mx=120;    //predetermined value of maximum blood pressure in mmHg
char mxchar[3];
double mn=80;     //predetermined value of minimum blood pressure in mmHg
char mnchar[3];
float freq=60;   //predetermined value of cardiac frequency in beats/min
char freqchar[3];
int xx, yy;
int a=0;
int inputplot;
int count=0;
int activalista=0;
int pRead;
double totalpress=0;
double kp=0.05;    //PID values
double ki=1.4;
double kd=0;
int num_puntos;
float CutOff=5.0;  //Cut off frecuency in Hz
float preal;
int pointsp;
float valuepreal=0;
float realpressure=0;

IIR::ORDER  order  = IIR::ORDER::OD2;

Filter f(CutOff, Tm/1000, order);   //lowpass Butterworth second order filter definition

double Setpoint, Input, Output;
PID myPID(&Input, &Output, &Setpoint,kp,ki,kd, DIRECT);  //PID definition


void setup(){
  
Serial.begin (115200);
lcd.InitLCD();
lcd.clrScr();
file.initFAT();
myTouch.InitTouch(TOUCH_ORIENTATION);
myTouch.setPrecision(PREC_MEDIUM);
lcd.setFont(BigFont);

lcd.setColor(0,255,255);
lcd.print("Unitat De Biofisica",CENTER,140);
lcd.print("Universitat de Barcelona",CENTER,180);
delay(2000);

  
myPID.SetMode(AUTOMATIC);

for (pointsp=0; pointsp<100; pointsp++){
  preal=analogRead(1);
  valuepreal=valuepreal+preal;
}
realpressure=valuepreal/100;

num_puntos=(60/(freq*Tm/1000))+0.5;   //calculate the number of points in one cycle for a determinate frequency

for (i=0; i<num_puntos;i++){
   
          
         lista[i] = 0;    //fullfil the array with 0, as many as the needed number of points 
  
         
          }
 
i=0;

inicio();  //function used to set the main screen

}

void loop(){



  while (myTouch.dataAvailable() == true)
  { 
  
     myTouch.read();       //get the x and y coordinates of the touched zone
     xx = myTouch.getX();  
     yy = myTouch.getY();


    if ((mx>=mn) && (mx<=180)){    //define the threshold for the maximum values
      
          if ((xx>=560) && (xx<=610) && (yy>20) && (yy<40)){  //if the upper arrow is touched, increase the maximum pressure value
            lcd.setColor(WHITE);
  
            mx=mx+0.1;
            dtostrf(mx,3,0,mxchar);
            lcd.print(mxchar,133,430);
            mx=mx;
            }


          if ((xx>=560) && (xx<=610) && (yy>0) && (yy<20)){  //if the lower arrow is touched, decrease the maximum pressure value
             lcd.setColor(WHITE);
             mx=mx-0.1;
             dtostrf(mx,3,0,mxchar);
             lcd.print(mxchar,133,430);
             mx=mx;
           }
    }
    
     if (mx<mn){mx=mn;}     //if the value exceeds the threshold, remain in the limit value
     if (mx>180){mx=180;}
     
       
      
     if ((mn>=40) && (mn<=mx)){  

          if ((xx>=300) && (xx<=350) && (yy>20) && (yy<40)){
            lcd.setColor(WHITE); 
            mn=mn+0.1;
            dtostrf(mn,3,0,mnchar);
            lcd.print(mnchar,390,430);
            mn=mn;
           }
      

         if ((xx>=300) && (xx<=350) && (yy>0) && (yy<20)){
             lcd.setColor(WHITE);
             mn=mn-0.1;
             dtostrf(mn,3,0,mnchar);
             lcd.print(mnchar,390,430);
             mn=mn;
            }
    }


      if (mn<40){mn=40;}
      if (mn>mx){mn=mx;}
      
 
     
    if ((freq>=60) && (freq<=120)){
      
         if ((xx>=50) && (xx<=100) && (yy>20) && (yy<40)){
             lcd.setColor(WHITE); 
             freq=freq+0.1;
             dtostrf(freq,3,0,freqchar);
             lcd.print(freqchar,640,430);
             freq=freq;
             }
      

         if ((xx>=50) && (xx<=100) && (yy>0) && (yy<20)){
              lcd.setColor(WHITE);
              freq=freq-0.1;
              dtostrf(freq,3,0,freqchar);
              lcd.print(freqchar,640,430);
              freq=freq;
              }
    }

     if (freq<60){freq=60;}
     if (freq>120){freq=120;}

     

  if ((xx>=150) && (xx<=240) && (yy>=0) && (yy<=20)){     //to change the frequency value, the green button needs to be pressed
  activalista=0;
  
  }


}

 
if (activalista==0){     //if the green button is pressed, then fullfil the array with the values generated by the wave equation

    num_puntos=(60/(freq*Tm/1000))+0.5;  //number of points needed to complete the array

     for (i=0; i<num_puntos;i++){
   
          y= mu+a1*cos(2*3.14*(freq/60)*t)+a2*cos(4*3.14*(freq/60)*t+phase); //blood pressure wave 
          lista[i] = y; 
  
                              
            t= t+(Tm/1000);    

      maxValor = max(maxValor,lista[i]);  //get the maximum and minimum values of the array
      minValor = min(minValor,lista[i]); 
         
           }
  
  i=0;

  t=0;
  
  activalista=1;  //stop generating the array with the values, keep the same array until the frequency gets changed
      
  }


 
  pRead=analogRead(0);  //pressure read by the sensor
  totalpress=totalpress+pRead;   
  count=count+1;

  
  if ((millis()-t0)>=Tm){
 
    t0=millis();
      
    if (i>=num_puntos) {i=0;}

        int mx_map=(mx/200)*1023;    //convert the maximum and minimum pressure values into numbers between 0-1023

        int mn_map=mn/200*1023;   

        consigna=  map(lista[i], minValor, maxValor, mn_map, mx_map);  //readjust each value considering the maximum and minimum pressure values introduced 
     
        int out_consigna=map(consigna,0,1023,0,255); //as this value will be sent as an output, it needs to be ranged between 0 - 255
     
      
        analogWrite(8,out_consigna);
     
        Setpoint=consigna;                    
        Input = totalpress/(count+1); //average of all values read every 10 ms
    
     myPID.Compute();
     analogWrite(9,Output);
     
     float error =((consigna-Input));

     Serial.print(consigna*200/1023);
     Serial.print(",");
     Serial.println(Input*200/1023);
     
     count=0;
     totalpress=0;
     
     inputplot=map(Input,0,920,372,45); //readjust the values to match them with the appropiate pixels
     
     float filteredval = f.filterIn(inputplot); //filter the signal with a Butterwoth second order low past filter
    


      if ((a>=37) && (a<=785)){       //graph the signal  (time vs pressure)                          
                                       
                                         
                                         lcd.setColor(BLACK);
                                         lcd.drawPixel(a,inputplotlast[a-37]);  //remove the old signal while the new one comes out
                                         lcd.setColor(WHITE);   
                                         lcd.drawPixel(a,filteredval);   
                                         inputplotlast[a-37]=filteredval;
                                         a=a+1;
                                         }
     else {a=37;}
      
    i=i+1;


 }

      
}
  
  
////////////////////////////////////////////
 
void inicio(){
       
          lcd.clrScr();
          lcd.setColor(RED);
          lcd.drawLine(35,30,35,372); //vertical axis
                                      
                     for (int i=30; i <=370; i=i+19){
                                lcd.drawLine(30,i,40,i);  //ticks y axis
                             
                                                 }

                     lcd.setColor(CYAN);                 // longitudinal lines         
                     for (int l=30; l<=370; l=l+38){  
                      int contador=0;
                      
                      while (contador<740){
                        lcd.drawPixel(50+contador,l);
                        contador=contador+10;
                        
                      }
         
                     }
                                                

            lcd.setColor(RED);           // y axis labels                 
            lcd.setFont(SmallFont) ;                                 
            lcd.print("0",10,365);
            lcd.print("20",10,327);
            lcd.print("40",10,289);
            lcd.print("60",10,251);
            lcd.print("80",10,213);
            lcd.print("100",5,175);
            lcd.print("120",5,137);
            lcd.print("140",5,99);
            lcd.print("160",5,61);
            lcd.print("180",5,23);
            lcd.print("P(mmHg)",25,5);
        
                                      
         lcd.drawLine(35,372,785,372); //horizontal axis
 
                    for (int x=35; x <=785; x=x+25){
                                lcd.drawLine(x,367,x,377);//ticks x axis
                                
                                                    }
          lcd.print("t(s)",760,345); // x axis labels
          lcd.print("0.5",77,380);
          lcd.print("1",132,380);
          lcd.print("1.5",177,380);
          lcd.print("2",232,380);
          lcd.print("2.5",277,380);
          lcd.print("3",332,380);
          lcd.print("3.5",377,380);
          lcd.print("4",432,380);
          lcd.print("4.5",477,380);
          lcd.print("5",532,380);
          lcd.print("5.5",575,380);
          lcd.print("6",632,380);
          lcd.print("6.5",675,380);
          lcd.print("7",732,380);
          lcd.print("7.5",770,380);


          lcd.setColor(WHITE);          //draw the boxes with the correspoding arrows
          lcd.drawRect(192,400,238,470); 
          lcd.drawLine(192,435,238,435);
          lcd.drawLine(206,425,215,405);
          lcd.drawLine(206,425,223,425);
          lcd.drawLine(215,405,223,425);
          lcd.drawLine(206,445,223,445);
          lcd.drawLine(206,445,215,465);
          lcd.drawLine(215,465,223,445);

          lcd.drawRect(452,400,498,470);
          lcd.drawLine(452,435,498,435);
          lcd.drawLine(466,425,475,405);
          lcd.drawLine(466,425,483,425);
          lcd.drawLine(475,405,483,425);
          lcd.drawLine(466,445,483,445);
          lcd.drawLine(466,445,475,465);
          lcd.drawLine(475,465,483,445);

          lcd.drawRect(702,400,748,470);
          lcd.drawLine(702,435,748,435);
          lcd.drawLine(716,425,725,405);
          lcd.drawLine(716,425,733,425);
          lcd.drawLine(725,405,733,425);
          lcd.drawLine(716,445,733,445);
          lcd.drawLine(716,445,725,465);
          lcd.drawLine(725,465,733,445);
          
          lcd.setFont(BigFont);
          dtostrf(mx,3,0,mxchar);
          lcd.print(mxchar,133,430);                           
          dtostrf(mn,3,0,mnchar);
          lcd.print(mnchar,390,430); 
          dtostrf(freq,3,0,freqchar);
          lcd.print(freqchar,640,430);
          
          
          lcd.setColor(YELLOW);
          lcd.print("Pmax:",40,430);
          lcd.print("Pmin:",300,430); 
          lcd.setColor(GREEN);
          lcd.drawRoundRect(565,425,625,455);
          lcd.print("FC:",575,430);

                               
     }

//////////////////////////////////////////////////////////////
